@@ -12,6 +12,7 @@ from tempfile import TemporaryFile
 
 import pytest
 from _pytest.compat import CaptureIO
+from _pytest.fixtures import FixtureRequest
 
 patchsysdict = {0: "stdin", 1: "stdout", 2: "stderr"}
 
@@ -241,13 +242,12 @@ class CaptureManager:
 capture_fixtures = {"capfd", "capfdbinary", "capsys", "capsysbinary"}
 
 
-def _ensure_only_one_capture_fixture(request, name):
-    fixtures = set(request.fixturenames) & capture_fixtures - {name}
+def _ensure_only_one_capture_fixture(request: FixtureRequest, name):
+    fixtures = sorted(set(request.fixturenames) & capture_fixtures - {name})
     if fixtures:
-        fixtures = sorted(fixtures)
-        fixtures = fixtures[0] if len(fixtures) == 1 else fixtures
+        arg = fixtures[0] if len(fixtures) == 1 else fixtures
         raise request.raiseerror(
-            "cannot use {} and {} at the same time".format(fixtures, name)
+            "cannot use {} and {} at the same time".format(arg, name)
         )
 
 
@@ -693,17 +693,12 @@ class SysCaptureBinary(SysCapture):
 
 
 class DontReadFromInput:
-    """Temporary stub class.  Ideally when stdin is accessed, the
-    capturing should be turned off, with possibly all data captured
-    so far sent to the screen.  This should be configurable, though,
-    because in automated test runs it is better to crash than
-    hang indefinitely.
-    """
-
     encoding = None
 
     def read(self, *args):
-        raise IOError("reading from stdin while output is captured")
+        raise IOError(
+            "pytest: reading from stdin while output is captured!  Consider using `-s`."
+        )
 
     readline = read
     readlines = read
