@@ -4,12 +4,20 @@ python version compatibility code
 import functools
 import inspect
 import io
+import os
 import re
 import sys
 from contextlib import contextmanager
 from inspect import Parameter
 from inspect import signature
+from typing import Any
+from typing import Callable
+from typing import Generic
+from typing import Optional
 from typing import overload
+from typing import Tuple
+from typing import TypeVar
+from typing import Union
 
 import attr
 import py
@@ -19,6 +27,13 @@ from _pytest._io.saferepr import saferepr
 from _pytest.outcomes import fail
 from _pytest.outcomes import TEST_OUTCOME
 
+if False:  # TYPE_CHECKING
+    from typing import Type  # noqa: F401 (used in type string)
+
+
+_T = TypeVar("_T")
+_S = TypeVar("_S")
+
 
 NOTSET = object()
 
@@ -28,12 +43,13 @@ MODULE_NOT_FOUND_ERROR = (
 
 
 if sys.version_info >= (3, 8):
-    from importlib import metadata as importlib_metadata  # noqa: F401
+    # Type ignored until next mypy release.
+    from importlib import metadata as importlib_metadata  # type: ignore
 else:
     import importlib_metadata  # noqa: F401
 
 
-def _format_args(func):
+def _format_args(func: Callable[..., Any]) -> str:
     return str(signature(func))
 
 
@@ -41,12 +57,25 @@ def _format_args(func):
 REGEX_TYPE = type(re.compile(""))
 
 
-def is_generator(func):
+if sys.version_info < (3, 6):
+
+    def fspath(p):
+        """os.fspath replacement, useful to point out when we should replace it by the
+        real function once we drop py35.
+        """
+        return str(p)
+
+
+else:
+    fspath = os.fspath
+
+
+def is_generator(func: object) -> bool:
     genfunc = inspect.isgeneratorfunction(func)
     return genfunc and not iscoroutinefunction(func)
 
 
-def iscoroutinefunction(func):
+def iscoroutinefunction(func: object) -> bool:
     """
     Return True if func is a coroutine function (a function defined with async
     def syntax, and doesn't contain yield), or a function decorated with
@@ -59,7 +88,7 @@ def iscoroutinefunction(func):
     return inspect.iscoroutinefunction(func) or getattr(func, "_is_coroutine", False)
 
 
-def getlocation(function, curdir=None):
+def getlocation(function, curdir=None) -> str:
     function = get_real_func(function)
     fn = py.path.local(inspect.getfile(function))
     lineno = function.__code__.co_firstlineno
@@ -68,7 +97,7 @@ def getlocation(function, curdir=None):
     return "%s:%d" % (fn, lineno + 1)
 
 
-def num_mock_patch_args(function):
+def num_mock_patch_args(function) -> int:
     """ return number of arguments used up by mock arguments (if any) """
     patchings = getattr(function, "patchings", None)
     if not patchings:
@@ -87,7 +116,13 @@ def num_mock_patch_args(function):
     )
 
 
-def getfuncargnames(function, *, name: str = "", is_method=False, cls=None):
+def getfuncargnames(
+    function: Callable[..., Any],
+    *,
+    name: str = "",
+    is_method: bool = False,
+    cls: Optional[type] = None
+) -> Tuple[str, ...]:
     """Returns the names of a function's mandatory arguments.
 
     This should return the names of all function arguments that:
@@ -155,7 +190,7 @@ else:
     from contextlib import nullcontext  # noqa
 
 
-def get_default_arg_names(function):
+def get_default_arg_names(function: Callable[..., Any]) -> Tuple[str, ...]:
     # Note: this code intentionally mirrors the code at the beginning of getfuncargnames,
     # to get the arguments which were excluded from its result because they had default values
     return tuple(
@@ -174,18 +209,18 @@ _non_printable_ascii_translate_table.update(
 )
 
 
-def _translate_non_printable(s):
+def _translate_non_printable(s: str) -> str:
     return s.translate(_non_printable_ascii_translate_table)
 
 
 STRING_TYPES = bytes, str
 
 
-def _bytes_to_ascii(val):
+def _bytes_to_ascii(val: bytes) -> str:
     return val.decode("ascii", "backslashreplace")
 
 
-def ascii_escaped(val):
+def ascii_escaped(val: Union[bytes, str]):
     """If val is pure ascii, returns it as a str().  Otherwise, escapes
     bytes objects into a sequence of escaped bytes:
 
@@ -282,7 +317,7 @@ def getimfunc(func):
         return func
 
 
-def safe_getattr(object, name, default):
+def safe_getattr(object: Any, name: str, default: Any) -> Any:
     """ Like getattr but return default upon any Exception or any OutcomeException.
 
     Attribute access can potentially fail for 'evil' Python objects.
@@ -296,7 +331,7 @@ def safe_getattr(object, name, default):
         return default
 
 
-def safe_isclass(obj):
+def safe_isclass(obj: object) -> bool:
     """Ignore any exception via isinstance on Python 3."""
     try:
         return inspect.isclass(obj)
@@ -317,39 +352,26 @@ COLLECT_FAKEMODULE_ATTRIBUTES = (
 )
 
 
-def _setup_collect_fakemodule():
+def _setup_collect_fakemodule() -> None:
     from types import ModuleType
     import pytest
 
-    pytest.collect = ModuleType("pytest.collect")
-    pytest.collect.__all__ = []  # used for setns
+    # Types ignored because the module is created dynamically.
+    pytest.collect = ModuleType("pytest.collect")  # type: ignore
+    pytest.collect.__all__ = []  # type: ignore  # used for setns
     for attr_name in COLLECT_FAKEMODULE_ATTRIBUTES:
-        setattr(pytest.collect, attr_name, getattr(pytest, attr_name))
+        setattr(pytest.collect, attr_name, getattr(pytest, attr_name))  # type: ignore
 
 
 class CaptureIO(io.TextIOWrapper):
-    def __init__(self):
+    def __init__(self) -> None:
         super().__init__(io.BytesIO(), encoding="UTF-8", newline="", write_through=True)
 
-    def getvalue(self):
+    def getvalue(self) -> str:
+        assert isinstance(self.buffer, io.BytesIO)
         return self.buffer.getvalue().decode("UTF-8")
 
 
-class FuncargnamesCompatAttr:
-    """ helper class so that Metafunc, Function and FixtureRequest
-    don't need to each define the "funcargnames" compatibility attribute.
-    """
-
-    @property
-    def funcargnames(self):
-        """ alias attribute for ``fixturenames`` for pre-2.3 compatibility"""
-        import warnings
-        from _pytest.deprecated import FUNCARGNAMES
-
-        warnings.warn(FUNCARGNAMES, stacklevel=2)
-        return self.fixturenames
-
-
 if sys.version_info < (3, 5, 2):  # pragma: no cover
 
     def overload(f):  # noqa: F811
@@ -360,3 +382,35 @@ if getattr(attr, "__version_info__", ()) >= (19, 2):
     ATTRS_EQ_FIELD = "eq"
 else:
     ATTRS_EQ_FIELD = "cmp"
+
+
+if sys.version_info >= (3, 8):
+    # TODO: Remove type ignore on next mypy update.
+    # https://github.com/python/typeshed/commit/add0b5e930a1db16560fde45a3b710eefc625709
+    from functools import cached_property  # type: ignore
+else:
+
+    class cached_property(Generic[_S, _T]):
+        __slots__ = ("func", "__doc__")
+
+        def __init__(self, func: Callable[[_S], _T]) -> None:
+            self.func = func
+            self.__doc__ = func.__doc__
+
+        @overload
+        def __get__(
+            self, instance: None, owner: Optional["Type[_S]"] = ...
+        ) -> "cached_property[_S, _T]":
+            raise NotImplementedError()
+
+        @overload  # noqa: F811
+        def __get__(  # noqa: F811
+            self, instance: _S, owner: Optional["Type[_S]"] = ...
+        ) -> _T:
+            raise NotImplementedError()
+
+        def __get__(self, instance, owner=None):  # noqa: F811
+            if instance is None:
+                return self
+            value = instance.__dict__[self.func.__name__] = self.func(instance)
+            return value
