@@ -1,5 +1,8 @@
+from io import StringIO
 from pprint import pprint
+from typing import List
 from typing import Optional
+from typing import Tuple
 from typing import Union
 
 import py
@@ -14,6 +17,7 @@ from _pytest._code.code import ReprFuncArgs
 from _pytest._code.code import ReprLocals
 from _pytest._code.code import ReprTraceback
 from _pytest._code.code import TerminalRepr
+from _pytest.nodes import Node
 from _pytest.outcomes import skip
 from _pytest.pathlib import Path
 
@@ -32,14 +36,17 @@ def getslaveinfoline(node):
 
 class BaseReport:
     when = None  # type: Optional[str]
-    location = None
+    location = None  # type: Optional[Tuple[str, Optional[int], str]]
+    longrepr = None
+    sections = []  # type: List[Tuple[str, str]]
+    nodeid = None  # type: str
 
     def __init__(self, **kw):
         self.__dict__.update(kw)
 
-    def toterminal(self, out):
+    def toterminal(self, out) -> None:
         if hasattr(self, "node"):
-            out.line(getslaveinfoline(self.node))
+            out.line(getslaveinfoline(self.node))  # type: ignore
 
         longrepr = self.longrepr
         if longrepr is None:
@@ -180,7 +187,7 @@ class BaseReport:
 
 def _report_unserialization_failure(type_name, report_class, reportdict):
     url = "https://github.com/pytest-dev/pytest/issues"
-    stream = py.io.TextIO()
+    stream = StringIO()
     pprint("-" * 100, stream=stream)
     pprint("INTERNALERROR: Unknown entry type returned: %s" % type_name, stream=stream)
     pprint("report_name: %s" % report_class, stream=stream)
@@ -200,7 +207,7 @@ class TestReport(BaseReport):
     def __init__(
         self,
         nodeid,
-        location,
+        location: Tuple[str, Optional[int], str],
         keywords,
         outcome,
         longrepr,
@@ -209,14 +216,14 @@ class TestReport(BaseReport):
         duration=0,
         user_properties=None,
         **extra
-    ):
+    ) -> None:
         #: normalized collection node id
         self.nodeid = nodeid
 
         #: a (filesystempath, lineno, domaininfo) tuple indicating the
         #: actual location of a test item - it might be different from the
         #: collected one e.g. if a method is inherited from a different module.
-        self.location = location
+        self.location = location  # type: Tuple[str, Optional[int], str]
 
         #: a name -> value dictionary containing all keywords and
         #: markers associated with a test invocation.
@@ -299,7 +306,9 @@ class TestReport(BaseReport):
 class CollectReport(BaseReport):
     when = "collect"
 
-    def __init__(self, nodeid, outcome, longrepr, result, sections=(), **extra):
+    def __init__(
+        self, nodeid: str, outcome, longrepr, result: List[Node], sections=(), **extra
+    ) -> None:
         self.nodeid = nodeid
         self.outcome = outcome
         self.longrepr = longrepr
@@ -321,25 +330,25 @@ class CollectErrorRepr(TerminalRepr):
     def __init__(self, msg):
         self.longrepr = msg
 
-    def toterminal(self, out):
+    def toterminal(self, out) -> None:
         out.line(self.longrepr, red=True)
 
 
 def pytest_report_to_serializable(report):
     if isinstance(report, (TestReport, CollectReport)):
         data = report._to_json()
-        data["_report_type"] = report.__class__.__name__
+        data["$report_type"] = report.__class__.__name__
         return data
 
 
 def pytest_report_from_serializable(data):
-    if "_report_type" in data:
-        if data["_report_type"] == "TestReport":
+    if "$report_type" in data:
+        if data["$report_type"] == "TestReport":
             return TestReport._from_json(data)
-        elif data["_report_type"] == "CollectReport":
+        elif data["$report_type"] == "CollectReport":
             return CollectReport._from_json(data)
         assert False, "Unknown report_type unserialize data: {}".format(
-            data["_report_type"]
+            data["$report_type"]
         )
 
 
@@ -471,7 +480,9 @@ def _report_kwargs_from_json(reportdict):
                         description,
                     )
                 )
-            exception_info = ExceptionChainRepr(chain)
+            exception_info = ExceptionChainRepr(
+                chain
+            )  # type: Union[ExceptionChainRepr,ReprExceptionInfo]
         else:
             exception_info = ReprExceptionInfo(reprtraceback, reprcrash)
 
