@@ -1,9 +1,4 @@
-# -*- coding: utf-8 -*-
 """ discovery and running of std-library "unittest" style tests. """
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
-
 import sys
 import traceback
 
@@ -11,11 +6,13 @@ import _pytest._code
 import pytest
 from _pytest.compat import getimfunc
 from _pytest.config import hookimpl
+from _pytest.outcomes import exit
 from _pytest.outcomes import fail
 from _pytest.outcomes import skip
 from _pytest.outcomes import xfail
 from _pytest.python import Class
 from _pytest.python import Function
+from _pytest.runner import CallInfo
 
 
 def pytest_pycollect_makeitem(collector, name, obj):
@@ -113,25 +110,13 @@ class TestCaseFunction(Function):
 
     def setup(self):
         self._testcase = self.parent.obj(self.name)
-        self._fix_unittest_skip_decorator()
+        self._obj = getattr(self._testcase, self.name)
         if hasattr(self, "_request"):
             self._request._fillfixtures()
 
-    def _fix_unittest_skip_decorator(self):
-        """
-        The @unittest.skip decorator calls functools.wraps(self._testcase)
-        The call to functools.wraps() fails unless self._testcase
-        has a __name__ attribute. This is usually automatically supplied
-        if the test is a function or method, but we need to add manually
-        here.
-
-        See issue #1169
-        """
-        if sys.version_info[0] == 2:
-            setattr(self._testcase, "__name__", self.name)
-
     def teardown(self):
         self._testcase = None
+        self._obj = None
 
     def startTest(self, testcase):
         pass
@@ -170,6 +155,11 @@ class TestCaseFunction(Function):
         self.__dict__.setdefault("_excinfo", []).append(excinfo)
 
     def addError(self, testcase, rawexcinfo):
+        try:
+            if isinstance(rawexcinfo[1], exit.Exception):
+                exit(rawexcinfo[1].msg)
+        except TypeError:
+            pass
         self._addexcinfo(rawexcinfo)
 
     def addFailure(self, testcase, rawexcinfo):
@@ -208,12 +198,7 @@ class TestCaseFunction(Function):
             skip_why = getattr(
                 self._testcase.__class__, "__unittest_skip_why__", ""
             ) or getattr(testMethod, "__unittest_skip_why__", "")
-            try:  # PY3, unittest2 on PY2
-                self._testcase._addSkip(self, self._testcase, skip_why)
-            except TypeError:  # PY2
-                if sys.version_info[0] != 2:
-                    raise
-                self._testcase._addSkip(self, skip_why)
+            self._testcase._addSkip(self, self._testcase, skip_why)
             return True
         return False
 
@@ -245,6 +230,14 @@ def pytest_runtest_makereport(item, call):
             except AttributeError:
                 pass
 
+    unittest = sys.modules.get("unittest")
+    if unittest and call.excinfo and call.excinfo.errisinstance(unittest.SkipTest):
+        # let's substitute the excinfo with a pytest.skip one
+        call2 = CallInfo.from_call(
+            lambda: pytest.skip(str(call.excinfo.value)), call.when
+        )
+        call.excinfo = call2.excinfo
+
 
 # twisted trial support
 
